package mohwhelper

// WriteoffRequest 是 writeoff 的 Request
type WriteoffRequest struct {
	UnitNo       string `json:"unitNo"`       // 服務單位代碼
	RequestAt    string `json:"requestDt"`    // 申請日期時間
	SourceSystem string `json:"sourceSystem"` // 來源系統別
	ApData       string `json:"apdata"`       // 申請內容
	Checksum     string `json:"checksum"`     // 校驗碼
}

// WriteoffResponse 是 writeoff 的 Response
type WriteoffResponse struct {
	ReturnCode   int    `json:"rtncode"`    // 回覆結果代碼
	ResponseAt   string `json:"responseDt"` // 回覆日期時間
	ErrorMessage string `json:"errmsg"`     // 回覆訊息
	Result       string `json:"result"`     // 回覆明細
	Checksum     string `json:"checksum"`   // 校驗碼
	ReturnSeq    string `json:"rtnSeq"`     // 回覆序號
	Message      string `json:"message"`    // 回覆訊息
}

type FeeAuditInfo struct {
	WriteoffYYYYMM string `json:"writeoff_yyyymm"` // 支審年月
	TransNo        string `json:"trans_no"`        // 交易序號
}

// FeeApplyUpload 是 FeeApply 的傳送資料
type FeeApplyUpload struct {
	ApplyInfo ApplyInfo `json:"apply_info"` // 申報資料
}

// ApplyInfo 是 FeeApplyUpload 的單筆資料
type ApplyInfo struct {
	FeeAuditInfo
	Records        int             `json:"records"`          // 服務記錄筆數
	Amount         int             `json:"amount"`           // 服務記錄金額
	Cases          int             `json:"cases"`            // 申請個案數
	CaseSvcRecords []CaseSvcRecord `json:"case_svc_records"` // 個案服務記錄
}

// CaseSvcRecord 是個案服務記錄
type CaseSvcRecord struct {
	ObjID             string  `json:"objid"`                         // 識別碼
	IDN               string  `json:"idn,omitempty"`                 // 個案身份證字號
	SvcAt             string  `json:"svc_dt,omitempty"`              // 服務日期
	GovItemCode       string  `json:"gov_item_cd,omitempty"`         // 照顧組合代碼
	SvcFeeType        string  `json:"svc_fee_tp,omitempty"`          // 服務類別
	Price             int     `json:"price,omitempty"`               // 單價
	Amount            int     `json:"amount,omitempty"`              // 數量
	SvcUserNo1        string  `json:"svc_user_no1,omitempty"`        // 照顧服務員帳號1
	SvcUserNo2        string  `json:"svc_user_no2,omitempty"`        // 照顧服務員帳號2
	SvcUserNo3        string  `json:"svc_user_no3,omitempty"`        // 照顧服務員帳號3
	SvcUserNo4        string  `json:"svc_user_no4,omitempty"`        // 照顧服務員帳號4
	SvcUserNo5        string  `json:"svc_user_no5,omitempty"`        // 照顧服務員帳號5
	StartHH           string  `json:"start_hh,omitempty"`            // 起始時段-小時
	StartMM           string  `json:"start_mm,omitempty"`            // 起始時段-分鐘
	EndHH             string  `json:"end_hh,omitempty"`              // 結束時段-小時
	EndMM             string  `json:"end_mm,omitempty"`              // 結束時段-分鐘
	Remark            string  `json:"remark,omitempty"`              // 備註
	LastSvc           string  `json:"last_svc,omitempty"`            // 臨終日照顧
	Addr1             string  `json:"addr1,omitempty"`               // 出發地
	Addr2             string  `json:"addr2,omitempty"`               // 目的地
	CarNo             string  `json:"car_no,omitempty"`              // 車號
	Driver            string  `json:"driver,omitempty"`              // 駕駛員
	ApplyAA03         string  `json:"apply_aa03,omitempty"`          // 是否申報AA03
	SvcItem           string  `json:"svc_item,omitempty"`            // 服務項目
	SvcItemOther      string  `json:"svc_item_other,omitempty"`      // 服務項目-其他
	SvcPeople         string  `json:"svc_people,omitempty"`          // 服務對象
	SvcContent        string  `json:"svc_content,omitempty"`         // 服務內容
	SvcUnit           string  `json:"svc_unit,omitempty"`            // 提供專業服務單位
	ApplyAA09         string  `json:"apply_aa09,omitempty"`          // 是否申報AA09
	AA10Status        string  `json:"aa10_status,omitempty"`         // AA10申報狀態
	MissedVisit       string  `json:"missed_visit,omitempty"`        // 訪視/服務未遇
	SvcEndDt          string  `json:"svc_end_dt,omitempty"`          // 服務結束日期
	ApplyCovid19      string  `json:"apply_covid19,omitempty"`       // 是否為陪同施打COVID-19疫苗
	SvcPoint          string  `json:"svc_point,omitempty"`           // 服務重點
	SvcPointOther     string  `json:"svc_point_other,omitempty"`     // 服務重點-其他
	SvcTrace          string  `json:"svc_trace,omitempty"`           // 追蹤服務適應與介入情形
	SvcGoal           string  `json:"svc_goal,omitempty"`            // 各項服務目標及整體計畫目標達成情形
	SvcSuitable       string  `json:"svc_suitable,omitempty"`        // 服務適切性
	SvccGoalType      string  `json:"svcc_goal_type,omitempty"`      // 專業服務復能目標達成情形
	SvccGoal          string  `json:"svcc_goal,omitempty"`           // 專業服務復能目標
	SvccContentTarget string  `json:"svcc_content_target,omitempty"` // 專業服務指導對象
	SvccContent       string  `json:"svcc_content,omitempty"`        // 專業服務服務內容
	SvccSuggest       string  `json:"svcc_suggest,omitempty"`        // 專業服務指導建議摘要
	Addrlat1          float64 `json:"addrlat1,omitempty"`            // 出發地-緯度
	Addrlng1          float64 `json:"addrlng1,omitempty"`            // 出發地-經度
	Addrlat2          float64 `json:"addrlat2,omitempty"`            // 目的地-緯度
	Addrlng2          float64 `json:"addrlng2,omitempty"`            // 目的地-經度
	Milage            float64 `json:"milage,omitempty"`              // 里程
	Bd03Type          string  `json:"bd03_type,omitempty"`           // 社區式服務交通接送(BD03)服務使用類型
}

// FeeApplyResult 是 FeeApply 的 Response.result
type FeeApplyResult struct {
	ProcessNumber int           `json:"proc_num"`   // 處理筆數
	SuccessNumber int           `json:"succ_num"`   // 上傳成功筆數
	ErrorNumber   int           `json:"err_num"`    // 上傳失敗筆數
	ErrorRecords  []ErrorRecord `json:"err_record"` // 錯誤服務記錄
}

// ErrorRecord 是錯誤服務記錄
type ErrorRecord struct {
	ObjID      string      `json:"objid"`    // 識別碼
	ErrorInfos []ErrorInfo `json:"err_info"` // 錯誤訊息
}

// ErrorInfo 是錯誤訊息
type ErrorInfo struct {
	ErrorCode    string `json:"err_code"`    // 錯誤碼
	ErrorMessage string `json:"err_message"` // 錯誤原因
}

// ObjDelUpload 是 ObjDel 的傳送資料
type ObjDelUpload struct {
	SvcDelInfo SvcDelInfo `json:"svcdel_info"` // 刪除資料
}

// SvcDelInfo 是刪除資料
type SvcDelInfo struct {
	FeeAuditInfo
	Records        int             `json:"records"`          // 服務記錄筆數
	CaseSvcRecords []CaseSvcRecord `json:"case_svc_records"` // 個案服務記錄
}

// AppCompletionNoticeUpload 是 AppCompletionNotice 的傳送資料
type AppCompletionNoticeUpload struct {
	FinishInfo FinishInfo `json:"finish_info"` // 申報確認資料
}

// FinishInfo 是申報確認資料
type FinishInfo struct {
	FeeAuditInfo
	FinishCityInfos []FinishCityInfo `json:"city_info"` // 縣市案號資訊
}

type FinishCityInfo struct {
	CityCode    string       `json:"city_cd"`      // 縣市代碼
	CaseNoInfos []CaseNoInfo `json:"case_no_info"` // 案件資訊
}

// CaseNoInfo 是案件資訊
type CaseNoInfo struct {
	CaseNo string `json:"case_no"` // 案件編號
}

// CityInfo 是縣市案號資訊
type CityInfo struct {
	CityCode  string     `json:"city_cd"`    // 縣市代碼
	CaseInfos []CaseInfo `json:"case_infos"` // 案件資訊
}

type CaseInfo struct {
	CaseNo string `json:"case_no"`           // 案件編號
	Status string `json:"status,omitempty"`  // 核銷狀況
	DocVer string `json:"doc_ver,omitempty"` // 總表版次
	AccNum string `json:"acc_num,omitempty"` // 簽證編號
}

// AppResultQueryUpload 是 AppResultQuery 的傳送資料
type AppResultQueryUpload struct {
	AppResultQueryInfo AppResultQueryInfo `json:"app_result_query_info"` // 查詢輸入資訊
}

// AppResultQueryInfo 是查詢輸入資訊
type AppResultQueryInfo struct {
	FeeAuditInfo
	QueryType string `json:"query_type"`        // 查詢類別
	CityCode  string `json:"city_cd,omitempty"` // 縣市代碼
	CaseNo    string `json:"case_no,omitempty"` // 核銷案號
}

// AppResultQueryResult 是 AppResultQuery 的 Response.result
type AppResultQueryResult struct {
	CityInfos            []CityInfo          `json:"city_info,omitempty"`                 // 縣市案件資訊
	WebapiProcessInfos   []WebapiProcessInfo `json:"webapi_process_info,omitempty"`       // API 執行結果資料
	CaseNo               string              `json:"case_no,omitempty"`                   // 核銷案號
	Records              int                 `json:"records,omitempty"`                   // 服務記錄筆數
	Cases                int                 `json:"cases,omitempty"`                     // 個案數
	Amount               float64             `json:"amount,omitempty"`                    // 申請核銷金額
	AuditMan             string              `json:"audit_man,omitempty"`                 // 承辦人員
	AuditReason          string              `json:"audit_reason,omitempty"`              // 承辦審核意見
	DocVer               string              `json:"doc_ver,omitempty"`                   // 總表版次
	VerDate              string              `json:"ver_dt,omitempty"`                    // 版次時間
	ApproveCasesNum      int                 `json:"approve_cases_num,omitempty"`         // 核定個案數
	ApproveRecordCount   int                 `json:"approve_record_count,omitempty"`      // 核定服務記錄數
	ApproveFee           float64             `json:"approve_fee,omitempty"`               // 核定金額
	ASvcFee              float64             `json:"a_svc_fee,omitempty"`                 // 政策鼓勵金額
	TempPaymentStatus    string              `json:"temp_payment_status,omitempty"`       // 暫付申請狀態
	TempPaymentFee       int                 `json:"temp_payment_fee,omitempty"`          // 分案暫付金額
	TempPaymentDocUrl    string              `json:"temp_payment_doc_url,omitempty"`      // 暫付總表下載路徑
	CaseSummaryNoticeUrl string              `json:"case_summary_notice_url,omitempty"`   // 總表下載路徑
	CaseSvcListUrl       string              `json:"case_svc_list_url,omitempty"`         // 清冊路徑
	CaseSvcListExcelUrl  string              `json:"case_svc_list_excel_url,omitempty"`   // 清冊 EXCEL 下載路徑
	CaseASvcListUrl      string              `json:"case_a_svc_list_url,omitempty"`       // A 碼清冊路徑
	CaseASvcListExcelUrl string              `json:"case_a_svc_list_excel_url,omitempty"` // A 碼清冊 EXCEL 下載路徑
	CaseErrListUrl       string              `json:"case_err_list_url,omitempty"`         // 申請記錄不通過清冊路徑
	CaseErrListExcelUrl  string              `json:"case_err_list_excel_url,omitempty"`   // 申請記錄不通過 EXCEL 清冊路徑
	TransNos             []string            `json:"trans_nos,omitempty"`                 // 分案已處理之單號申請單單號
	IncInAcc             float64             `json:"inc_in_acc,omitempty"`                // 核增金額
	IncInReason          string              `json:"inc_in_reason,omitempty"`             // 核增原因
	DecInAcc             float64             `json:"dec_in_acc,omitempty"`                // 核減金額
	DecInReason          string              `json:"dec_in_reason,omitempty"`             //核減原因
	ApproveRecords       []ApproveRecord     `json:"approve_records,omitempty"`           // 審核通過服務記錄
	ErrorRecords         []ExceptionRecord   `json:"err_records,omitempty"`               // 錯誤服務記錄
	ASvcRecords          []ASvcRecord        `json:"a_svc_records"`                       // A 碼加成資料區
}

// WebapiProcessInfo 是 API 執行結果資料
type WebapiProcessInfo struct {
	Function               string            `json:"function"`                 // API Function
	SourceSystem           string            `json:"source_system"`            // 來源系統別
	TransNo                string            `json:"trans_no"`                 // 交易序號
	Status                 string            `json:"status"`                   // API 執行狀態
	BatchProcResult        string            `json:"batch_proc_result"`        // 批次處理結果
	BatchProcNum           int               `json:"batch_proc_num"`           // 批次處理筆數
	BatchSuccNum           int               `json:"batch_succ_num"`           // 批次處理成功筆數
	BatchErrNum            int               `json:"batch_err_num"`            // 批次處理失敗筆數
	ExceptionRecords       []ExceptionRecord `json:"exception_records"`        // 分案異常資料
	DeleteRecords          []string          `json:"delete_records"`           // 服務紀錄刪除成功資料
	DeleteExceptionRecords []ExceptionRecord `json:"delete_exception_records"` // 服務紀錄刪除失敗資料
}

// ExceptionRecord 是異常資料
type ExceptionRecord struct {
	SourceSystem string `json:"source_system,omitempty"` // 來源系統別
	ObjID        string `json:"objid,omitempty"`         // 識別碼
	TransNo      string `json:"trans_no,omitempty"`      // 交易序號
	ErrorCode    string `json:"err_code"`                // 錯誤碼
	ErrorMessage string `json:"err_message"`             // 錯誤原因
}

// ApproveRecord 是審核通過服務記錄
type ApproveRecord struct {
	SourceSystem string  `json:"source_system"` // 來源系統別
	ObjID        string  `json:"objid"`         // 識別碼
	TransNo      string  `json:"trans_no"`      // 交易序號
	Price        float64 `json:"price"`         // 單價
	Copayment    float64 `json:"copayment"`     // 自付額
}

// ASvcRecord 是 A 碼加成資料區
type ASvcRecord struct {
	RefSourceSystem string  `json:"ref_source_system"` // 審核通過服務記錄來源系統別
	RefObjID        string  `json:"ref_objid"`         // 審核通過服務記錄識別碼
	AGovItemCode    string  `json:"a_gov_item_cd"`     // A 碼
	Price           float64 `json:"price"`             // 單價
}

// AppCancelUpload 是 AppCancel 的傳送資料
type AppCancelUpload struct {
	CancelInfo CancelInfo `json:"cancel_info"` // 服務單位撤回資料

}

// CancelInfo 是服務單位撤回資料
type CancelInfo struct {
	FeeAuditInfo
	CityCode string `json:"city_cd,omitempty"` // 縣市代碼
	CaseNo   string `json:"case_no,omitempty"` // 核銷案號
}

// CancelResultResponseUpload 是 CancelResultResponse 的傳送資料
type CancelResultResponseUpload struct {
	CancelResultResponseInfo CancelResultResponseInfo `json:"CancelResultResponse_info"` // 取消交易單結果回報資料
}

// CancelResultResponseInfo 是取消交易單結果回報資料
type CancelResultResponseInfo struct {
	FeeAuditInfo
	CancelTransNo string `json:"cancel_trans_no"` // 所要取消結果回報之交易序號
}
